==> openclip-0.0.9.tar.gz: openclip-0.0.9/openclip/core.py <==
# -*- coding: utf-8 -*-
"""
    openclip/core.py
    =======================================

    provides the core `OpenClip` class which wraps all of
    the module functionality in one class/object
"""
import os
import uuid
import glob
import re
from datetime import datetime
from copy import deepcopy
from openclip.exceptions import *
import openclip.constants as _c
import openclip.util as util
from openclip.elements import Clip
from . import LOG
try:
    from lxml import etree as ET
except ImportError:
    try:
        # Python 2.5
        import xml.etree.cElementTree as ET
        LOG.warning("running with cElementTree on Python 2.5 (or higher)")
    except ImportError:
        try:
            import xml.etree.ElementTree as ET
            LOG.warning("running with ElementTree on Python 2.5  (or higher)")
        except ImportError:
            try:
                # normal cElementTree install
                import cElementTree as ET
                LOG.warning("running with cElementTree")
            except ImportError:
                try:
                    # normal ElementTree install
                    import elementtree.ElementTree as ET
                    LOG.warning("running with ElementTree")
                except ImportError:
                    LOG.critical("Failed to import ElementTree from any known place")

__all__ = [ 'OpenClip' ]


class OpenClip(object):
    """
    This is the main OpenClip class used for creating complete
    clip element trees. It provides a number of features:

    * load openclip XML file and parse into element tree of child objects
    with relevant functionality for each part of the tree

    * output full tree from these component parts

    * provides CRUD (Create, Read, Update, Delete) of all `etree.Elements` and `etree.SubElements`

    * retains all non-specific XML Eg. User Data `<userData>` tag

    * generate valid openclip XML without input

    * creates a backup file before writing any new XML file

    Usage:

    `import openclip`
    `myclip = openclip.OpenClip('/path/to/myclip.clip')`
    `print myclip.clip.xml`

    the above returns full XML tree of clip file (while also objectifying all it's component parts)
    """

    parser = ET.XMLParser(remove_blank_text=True) if util.lxml_installed() else ET.XMLParser()
    """the XML parser to use for extracting our data from XML files or strings
    if using lxml it will use the `remove_blank_text` attribute to more
    effectively format the data, removing blank text between tags (newlines etc.)"""

    def __init__(self, src_xml_path=None):
        self.src_xml_path = src_xml_path
        """source XML path from which we read the XML file
        this attribute is `None` by default"""
        self.basename = util.get_file_basename(src_xml_path)
        """basename of the file derived from the `src_xml_path` attribute
        custom setter and getter methods "clean" the input string
        also used as a fallback for file name and clip name"""
        if src_xml_path is None:
            self.dest_xml_path = util.get_cwd_or_tempdir()
            """destination to use for writing to"""
            self.clip = Clip(self.src_element)
            """base `Clip` object instance of `openclip.elements.Clip` class"""
        else:
            # get the actual file path to write to
            # dest_xml_path has custom setter/getter so...
            # ** this may return a new path in the system tempdir **
            self.dest_xml_path = self.src_xml_path
            """destination to use for writing to"""
            self.clip = Clip( self.src_element )
            """base `Clip` object instance of `openclip.elements.Clip` class"""

    def __unicode__(self):
        """
        this returns the object etree element as a unicode string
        """
        return util.element_to_xml(self.etree)

    def __str__(self):
        """
        strings need unicode entities replaced to display
        properly so we do that here
        """
        return self.__unicode__().encode('utf8', 'replace')

    @property
    def etree(self):
        """
        the clip `etree` attribute returns an `xml.Element` instance
        and this top level property wraps that `xml.Element` in
        an `ElementTree` for output
        """
        return ET.ElementTree(self.clip.etree)

    @property
    def root(self):
        """
        convenience property that returns the root Element from
        the OpenClip instance's etree
        """
        return self.etree.getroot()

    @property
    def xml(self):
        """
        convenience property that returns the XML string of
        the OpenClip instance's etree
        """
        return util.element_to_xml(self.etree)

    @property
    def dest_name(self):
        """
        `dest_name` attribute/property
        basename (with extension) of destination XML file
        """
        return '{file_name}{ext}'.format(file_name=self.basename, ext=_c.OPENCLIP_EXT)

    def _set_dest_xml_path(self, value):
        """
        overloaded setter method for the `dest_xml_path` attribute
        takes an input path string and checks if it is a valid path
        returning the current working directory or temp path if invalid
        """
        file_path = value
        # if file write status is None, file does not exist but path is writable
        if value is None or not util.clip_is_writable(value):
            LOG.warning('not a valid clip file path: {path}'.format(path=value))
            cwd = os.getcwd()
            file_path = os.path.join(cwd, self.dest_name)
            if not util.clip_is_writable(file_path):
                file_path = os.path.join(util.get_tempdir(), self.dest_name)
                LOG.info('new clip file destination set to: {path}'.format(path=file_path))
            else:
                LOG.info('new clip file destination set to: {path}'.format(path=file_path))
        self._dest_xml_path = file_path

    def _get_dest_xml_path(self):
        """
        returns destination xml file path
        """
        return self._dest_xml_path

    dest_xml_path = property(_get_dest_xml_path, _set_dest_xml_path, doc="destination path to write file to")
    """destination xml file path"""

    def create_etree(self):
        """
        generates and returns a basic OpenClip element clip
        """
        root_xml_fmt = '<?xml version="1.0" encoding="utf-8"?><clip type="clip" version="3"><name type="string">{clip_name}</name></clip>'
        root_xml = root_xml_fmt.format(clip_name=self.basename)
        root = ET.XML(root_xml)
        etree = ET.ElementTree(root)
        return etree

    def _get_src_element(self):
        """
        overloaded getter method for the src_element property
        parses a source XML file and if it fails for whatever reason
        returns an empty clip XML element

        > also resets the dest_xml_path if it fails so the source XML
        file is not overwritten unintentionally
        """
        try:
            try:
                # parse the document and return a clip file?
                src_element = ET.parse(self.src_xml_path, parser=OpenClip.parser).getroot()
            except ET.XMLSyntaxError as e:
                # not an XML file...
                raise InvalidOpenClipFile, e
            except IOError as e:
                # not a file!
                raise InvalidOpenClipFile, e
            if not ET.iselement(src_element) and not src_element.tag=='clip':
                # XML but NOT a clip file
                raise InvalidOpenClipFile, 'Cannot retrieve root <clip> Element.'
        except InvalidOpenClipFile as e:
            # ok so we tried but we'll have to init a new etree
            self.dest_xml_path = None
            LOG.debug(e)
            src_element = self.create_etree().getroot()
        except:
            self.dest_xml_path = None
            LOG.debug('Unknown Error parsing file... Does it exist?')
            LOG.warning('could not parse file. Generating generic clip XML without track or version.')
            src_element = self.create_etree().getroot()
        return src_element

    def _set_src_element(self, element):
        """
        overloaded setter method for the src_element property
        parses a source XML file and if it fails for whatever reason
        returns an empty clip XML element

        > also resets the dest_xml_path if it fails so the source XML
        file is not overwritten unintentionally
        """
        if not ET.iselement(element) and not element.tag=='clip':
            self._src_element = self.create_etree().getroot()
            self.dest_xml_path = None
            LOG.warning('rejected non-element or non-clip.')
        else:
            self._src_element = element

    src_element = property(_get_src_element, _set_src_element, doc="Source XML ElementTree/Document")
    """unmodified input xml.Element"""

    def write_clip_file(self):
        """
        creates a backup of the source XML file and
        writes XML output to file
        """
        with open(self.dest_xml_path, 'w') as outfile:
            outfile.write(self.xml)

==> openclip-0.0.12.tar.gz: openclip-0.0.12/openclip/core.py <==
# -*- coding: utf-8 -*-
""" The ``openclip.core`` module provides the ``OpenClip`` class which wraps all of
    the module functionality (element classes) in one class/object.
"""
import os
import uuid
import glob
import re
from datetime import datetime
from copy import deepcopy
from openclip.exceptions import *
import openclip.constants as _c
import openclip.util as util
from openclip.elements import Clip
from . import LOG
try:
    from lxml import etree as ET
except ImportError:
    try:
        # Python 2.5
        import xml.etree.cElementTree as ET
        LOG.warning("running with cElementTree on Python 2.5 (or higher)")
    except ImportError:
        try:
            import xml.etree.ElementTree as ET
            LOG.warning("running with ElementTree on Python 2.5  (or higher)")
        except ImportError:
            try:
                # normal cElementTree install
                import cElementTree as ET
                LOG.warning("running with cElementTree")
            except ImportError:
                try:
                    # normal ElementTree install
                    import elementtree.ElementTree as ET
                    LOG.warning("running with ElementTree")
                except ImportError:
                    LOG.critical("Failed to import ElementTree from any known place")

__all__ = [ 'OpenClip' ]


class OpenClip(object):
    """This is the main OpenClip class.

    The OpenClip class is used for creating complete
    clip element trees. It provides a number of features:

    * load openclip XML file and parse into element tree of child objects
      with relevant functionality for each part of the tree
    * output full tree from these component parts
    * provides **CRUD** (Create, Read, Update, Delete) of all ``etree.Elements`` and ``etree.SubElements``
    * retains all non-specific XML Eg. User Data ``<userData>`` tag
    * generate valid openclip XML without input
    * creates a backup file before writing any new XML file


    Example:
        The following example returns full XML tree of clip file
        (while also objectifying all it's component parts)

            >>> import openclip
            >>> myclip = openclip.OpenClip('/path/to/myclip.clip')
            >>> print myclip.clip.xml
            u'XML Output string here...'

    """

    parser = ET.XMLParser(remove_blank_text=True) if util.lxml_installed() else ET.XMLParser()
    """the XML parser to use for extracting our data from XML files or strings
    if using lxml it will use the ``remove_blank_text`` attribute to more
    effectively format the data, removing blank text between tags (newlines etc.)"""

    def __init__(self, src_xml_path=None):
        """Initialize the OpenClip instance.

        Args:
            src_xml_path (Optional[str]): path to a valid OpenClip XML file
        """
        self.src_xml_path = src_xml_path
        """source XML path from which we read the XML file
        this attribute is ``None`` by default"""
        self.basename = util.get_file_basename(src_xml_path)
        """basename of the file derived from the ``src_xml_path`` attribute
        custom setter and getter methods "clean" the input string
        also used as a fallback for file name and clip name"""
        if src_xml_path is None:
            self.dest_xml_path = util.get_cwd_or_tempdir()
            """destination to use for writing to"""
            self.clip = Clip(self.src_element)
            """base ``Clip`` object instance of ``openclip.elements.Clip`` class"""
        else:
            # get the actual file path to write to
            # dest_xml_path has custom setter/getter so...
            # ** this may return a new path in the system tempdir **
            self.dest_xml_path = self.src_xml_path
            """destination to use for writing to"""
            self.clip = Clip( self.src_element )
            """base ``Clip`` object instance of ``openclip.elements.Clip`` class"""

    def __unicode__(self):
        """
        this returns the object etree element as a unicode string
        """
        return util.element_to_xml(self.etree)

    def __str__(self):
        """
        strings need unicode entities replaced to display
        properly so we do that here
        """
        return self.__unicode__().encode('utf8', 'replace')

    @property
    def etree(self):
        """the clip ``etree`` attribute returns an ``xml.Element`` instance
        and this top level property wraps that ``xml.Element`` in
        an ``ElementTree`` for output
        """
        return ET.ElementTree(self.clip.etree)

    @property
    def root(self):
        """convenience property that returns the root Element from
        the OpenClip instance's etree
        """
        return self.etree.getroot()

    @property
    def xml(self):
        """convenience property that returns the XML string of
        the OpenClip instance's etree
        """
        return util.element_to_xml(self.etree)

    @property
    def dest_name(self):
        """basename (with extension) of destination XML file"""
        return '{file_name}{ext}'.format(file_name=self.basename, ext=_c.OPENCLIP_EXT)

    def _set_dest_xml_path(self, value):
        file_path = value
        # if file write status is None, file does not exist but path is writable
        if value is None or not util.clip_is_writable(value):
            LOG.warning('not a valid clip file path: {path}'.format(path=value))
            cwd = os.getcwd()
            file_path = os.path.join(cwd, self.dest_name)
            if not util.clip_is_writable(file_path):
                file_path = os.path.join(util.get_tempdir(), self.dest_name)
                LOG.info('new clip file destination set to: {path}'.format(path=file_path))
            else:
                LOG.info('new clip file destination set to: {path}'.format(path=file_path))
        self._dest_xml_path = file_path

    def _get_dest_xml_path(self):
        """This is the destination xml file path

        The overloaded setter takes an input path string and checks if it is a valid path
        returning the current working directory or temp path if invalid
        """
        return self._dest_xml_path

    dest_xml_path = property(_get_dest_xml_path, _set_dest_xml_path, doc="destination path to write file to")

    def create_etree(self):
        """generates and returns a basic OpenClip element clip
        """
        root_xml_fmt = '<?xml version="1.0" encoding="utf-8"?><clip type="clip" version="3"><name type="string">{clip_name}</name></clip>'
        root_xml = root_xml_fmt.format(clip_name=self.basename)
        root = ET.XML(root_xml)
        etree = ET.ElementTree(root)
        return etree

    def _get_src_element(self):
        """
        overloaded getter method for the src_element property
        parses a source XML file and if it fails for whatever reason
        returns an empty clip XML element

        Note:
            also resets the dest_xml_path if it fails so the source XML
            file is not overwritten unintentionally
        """
        try:
            try:
                # parse the document and return a clip file?
                src_element = ET.parse(self.src_xml_path, parser=OpenClip.parser).getroot()
            except ET.XMLSyntaxError as e:
                # not an XML file...
                raise InvalidOpenClipFile, e
            except IOError as e:
                # not a file!
                raise InvalidOpenClipFile, e
            if not ET.iselement(src_element) and not src_element.tag=='clip':
                # XML but NOT a clip file
                raise InvalidOpenClipFile, 'Cannot retrieve root <clip> Element.'
        except InvalidOpenClipFile as e:
            # ok so we tried but we'll have to init a new etree
            self.dest_xml_path = None
            LOG.debug(e)
            src_element = self.create_etree().getroot()
        except:
            self.dest_xml_path = None
            LOG.debug('Unknown Error parsing file... Does it exist?')
            LOG.warning('could not parse file. Generating generic clip XML without track or version.')
            src_element = self.create_etree().getroot()
        return src_element

    def _set_src_element(self, element):
        """overloaded setter method for the src_element property
        parses a source XML file and if it fails for whatever reason
        returns an empty clip XML element

        Note:
            also resets the dest_xml_path if it fails so the source XML
            file is not overwritten unintentionally
        """
        if not ET.iselement(element) and not element.tag=='clip':
            self._src_element = self.create_etree().getroot()
            self.dest_xml_path = None
            LOG.warning('rejected non-element or non-clip.')
        else:
            self._src_element = element

    src_element = property(_get_src_element, _set_src_element, doc="Source XML ElementTree/Document")
    """unmodified input xml.Element"""

    def write_clip_file(self):
        """creates a backup of the source XML file and
        writes XML output to file
        """
        with open(self.dest_xml_path, 'w') as outfile:
            outfile.write(self.xml)
